import { InjectionToken, ChangeDetectionStrategy, ChangeDetectorRef, Component, ElementRef, Inject, Input, Optional, PLATFORM_ID, Renderer2, Self, ViewChild, ViewEncapsulation, NgModule } from '@angular/core';
import 'codemirror/addon/display/placeholder';
import 'codemirror/addon/edit/continuelist';
import 'codemirror/mode/gfm/gfm';
import { fromTextArea } from 'codemirror';
import { Subject } from 'rxjs';
import { transition, trigger } from '@angular/animations';
import { coerceBooleanProperty, coerceNumberProperty } from '@angular/cdk/coercion';
import { isPlatformBrowser, CommonModule } from '@angular/common';
import { NgControl } from '@angular/forms';
import { fadeIn } from '@ng-tangram/components/core';
import { NtFormFieldControl } from '@ng-tangram/components/forms';
import { NtMarkdownBlockModule } from '@ng-tangram/pro/markdown-block';

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
/** @type {?} */
const NT_MARKDOWN_EDITOR_ICONS = new InjectionToken('nt-markdown-editor-icons');

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
/**
 * MarkdownEditor Commands
 */
// common regexp
/** @type {?} */
const LINE_REGEXP = /^([^\n]*)(\n|$)/gm;
// list regexp
/** @type {?} */
const UL_REGEXP = /^[*\-+]\s/;
/** @type {?} */
const UL_REGEXP_MULTIPLE = /^([*\-+]\s)([^\n]*)(\n|$)/gm;
/** @type {?} */
const OL_REGEXP = /^\d+\.\s+/;
/** @type {?} */
const OL_REGEXP_MULTIPLE = /^(\d+\.\s)([^\n]*)(\n|$)/gm;
/**
 * h1 - h6
 * @param {?} size header 大小值 {范围：1 - 6}
 * @param {?} doc CodeMirror.Doc 对象
 * @param {?} erase 是否要抹掉
 * @param {?=} multiple 是否是多行状态
 * @return {?}
 */
function header(size, doc, erase, multiple = false) {
    /** @type {?} */
    let single = new RegExp(`^#{${size}}\\s`);
    /** @type {?} */
    let multi = new RegExp(`^(#{${size}}\\s)([^\\n]*)(\\n|$)`, 'gm');
    /**
     * 输入光标位置
     * @type {?}
     */
    let start = doc.getCursor('start');
    /** @type {?} */
    let end = doc.getCursor('end');
    /**
     * 原文内容和替换的内容
     * @type {?}
     */
    let origin = doc.getRange({ line: start.line, ch: 0 }, end);
    /** @type {?} */
    let replacement = '';
    /** @type {?} */
    let symbol = Array(size).fill('#').join('');
    /** @type {?} */
    let startOffset = 0;
    /** @type {?} */
    let endOffset = 0;
    if (erase) {
        replacement = origin.replace(multi, (/**
         * @param {?} _match
         * @param {?} $1
         * @param {?} $2
         * @param {?} $3
         * @return {?}
         */
        (_match, $1, $2, $3) => {
            endOffset = $1.length;
            return `${$2}${$3}`;
        }));
        // 计算字符的移位数
        startOffset = doc.getLine(start.line).match(single) ? -size - 1 : 0;
        endOffset = doc.getLine(end.line).match(single) ? -endOffset : 0;
    }
    else {
        replacement = origin
            .replace(/^(#{1,6}\s)([^\n]*)(\n|$)/gm, (/**
         * @param {?} _match
         * @param {?} _$1
         * @param {?} $2
         * @param {?} $3
         * @return {?}
         */
        (_match, _$1, $2, $3) => `${$2}${$3}`))
            .replace(LINE_REGEXP, (/**
         * @param {?} match
         * @return {?}
         */
        match => !match.trim() && multiple ? match : `${symbol} ${match}`));
        // 计算字符的移位数
        startOffset = doc.getLine(start.line).match(single) ? 0 : size + 1;
        endOffset = doc.getLine(end.line).match(single) ? 0 : size + 1;
    }
    // 替换选中的文本
    doc.replaceRange(replacement, { line: start.line, ch: 0 }, end);
    // 重新选中内容, 如果是多行模式尾行的偏移独立计算
    doc.setSelection({ line: start.line, ch: start.ch + startOffset }, { line: end.line, ch: end.ch + (multiple ? endOffset : startOffset) }, { scroll: false });
}
/**
 * h1
 * @param {?} doc CodeMirror.Doc 对象
 * @param {?} erase 是否要抹掉
 * @param {?=} multiple 是否是多行状态
 * @return {?}
 */
function h1(doc, erase, multiple = false) {
    header(1, doc, erase, multiple);
}
/**
 * h2
 * @param {?} doc CodeMirror.Doc 对象
 * @param {?} erase 是否要抹掉
 * @param {?=} multiple 是否是多行状态
 * @return {?}
 */
function h2(doc, erase, multiple = false) {
    header(2, doc, erase, multiple);
}
/**
 * h3
 * @param {?} doc CodeMirror.Doc 对象
 * @param {?} erase 是否要抹掉
 * @param {?=} multiple 是否是多行状态
 * @return {?}
 */
function h3(doc, erase, multiple = false) {
    header(3, doc, erase, multiple);
}
/**
 * h4
 * @param {?} doc CodeMirror.Doc 对象
 * @param {?} erase 是否要抹掉
 * @param {?=} multiple 是否是多行状态
 * @return {?}
 */
function h4(doc, erase, multiple = false) {
    header(4, doc, erase, multiple);
}
/**
 * h5
 * @param {?} doc CodeMirror.Doc 对象
 * @param {?} erase 是否要抹掉
 * @param {?=} multiple 是否是多行状态
 * @return {?}
 */
function h5(doc, erase, multiple = false) {
    header(5, doc, erase, multiple);
}
/**
 * h6
 * @param {?} doc CodeMirror.Doc 对象
 * @param {?} erase 是否要抹掉
 * @param {?=} multiple 是否是多行状态
 * @return {?}
 */
function h6(doc, erase, multiple = false) {
    header(6, doc, erase, multiple);
}
/**
 * 无序列表命令
 * @param {?} doc CodeMirror.Doc 对象
 * @param {?} erase 是否要抹掉
 * @param {?=} multiple 是否是多行状态
 * @return {?}
 */
function ul(doc, erase, multiple = false) {
    /**
     * 输入光标位置
     * @type {?}
     */
    let start = doc.getCursor('start');
    /** @type {?} */
    let end = doc.getCursor('end');
    /**
     * 原文内容和替换的内容
     * @type {?}
     */
    let origin = doc.getRange({ line: start.line, ch: 0 }, end);
    /** @type {?} */
    let replacement = '';
    /** @type {?} */
    let startOffset = 0;
    /** @type {?} */
    let endOffset = 0;
    if (erase) {
        replacement = origin.replace(UL_REGEXP_MULTIPLE, (/**
         * @param {?} _match
         * @param {?} $1
         * @param {?} $2
         * @param {?} $3
         * @return {?}
         */
        (_match, $1, $2, $3) => {
            endOffset = $1.length;
            return `${$2}${$3}`;
        }));
        // 计算字符的移位数
        startOffset = doc.getLine(start.line).match(UL_REGEXP) ? -2 : 0;
        endOffset = doc.getLine(end.line).match(UL_REGEXP) ? -endOffset : 0;
    }
    else {
        replacement = origin.replace(LINE_REGEXP, (/**
         * @param {?} match
         * @return {?}
         */
        match => !match.trim() && multiple ? match : `- ${match}`));
        // 计算字符的移位数
        startOffset = doc.getLine(start.line).match(UL_REGEXP) ? 0 : 2;
        endOffset = doc.getLine(end.line).match(UL_REGEXP) ? 0 : 2;
    }
    // 替换选中的文本
    doc.replaceRange(replacement, { line: start.line, ch: 0 }, end);
    // 重新选中内容, 如果是多行模式尾行的偏移独立计算
    doc.setSelection({ line: start.line, ch: start.ch + startOffset }, { line: end.line, ch: end.ch + (multiple ? endOffset : startOffset) }, { scroll: false });
}
/**
 * 有序列表命令
 * @param {?} doc CodeMirror.Doc 对象
 * @param {?} erase 是否要抹掉
 * @param {?=} multiple 是否是多行状态
 * @return {?}
 */
function ol(doc, erase, multiple = false) {
    /**
     * 输入光标位置
     * @type {?}
     */
    let start = doc.getCursor('start');
    /** @type {?} */
    let end = doc.getCursor('end');
    /**
     * 原文内容和替换的内容
     * @type {?}
     */
    let origin = doc.getRange({ line: start.line, ch: 0 }, end);
    /** @type {?} */
    let replacement = '';
    /** @type {?} */
    let startOffset = 0;
    /** @type {?} */
    let index = 0;
    /** @type {?} */
    let endOffset = 0;
    if (erase) {
        replacement = origin.replace(OL_REGEXP_MULTIPLE, (/**
         * @param {?} _match
         * @param {?} $1
         * @param {?} $2
         * @param {?} $3
         * @return {?}
         */
        (_match, $1, $2, $3) => {
            startOffset = startOffset > 0 ? startOffset : $1.length;
            endOffset = $1.length;
            return `${$2}${$3}`;
        }));
        // 计算字符的移位数
        startOffset = doc.getLine(start.line).match(OL_REGEXP) ? -startOffset : 0;
        endOffset = doc.getLine(end.line).match(OL_REGEXP) ? -endOffset : 0;
    }
    else {
        replacement = origin.replace(LINE_REGEXP, (/**
         * @param {?} match
         * @return {?}
         */
        match => !match.trim() && multiple ? match : `${++index}. ${match}`));
        // 计算字符的移位数
        startOffset = doc.getLine(start.line).match(OL_REGEXP) ? 0 : 3;
        endOffset = doc.getLine(end.line).match(OL_REGEXP) ? 0 : index.toString().length + 2;
    }
    // 替换选中的文本
    doc.replaceRange(replacement, { line: start.line, ch: 0 }, end);
    // 重新选中内容, 如果是多行模式尾行的偏移独立计算
    doc.setSelection({ line: start.line, ch: start.ch + startOffset }, { line: end.line, ch: end.ch + (multiple ? endOffset : startOffset) }, { scroll: false });
}
/**
 * 链接
 * @param {?} doc CodeMirror.Doc 对象
 * @param {?} _erase
 * @param {?=} _multiple
 * @return {?}
 */
function link(doc, _erase, _multiple = false) {
    /** @type {?} */
    const start = doc.getCursor('start');
    /** @type {?} */
    const end = doc.getCursor('end');
    /** @type {?} */
    const selection = doc.getSelection() || '';
    /** @type {?} */
    const from = { line: start.line, ch: start.ch + 1 };
    /** @type {?} */
    const to = { line: end.line, ch: end.ch + (start.line === end.line ? 1 : 0) };
    doc.replaceRange(`[${selection}](http://)`, start, end, selection);
    doc.setSelection(from, to, { scroll: false });
}
/**
 * 图片
 * @param {?} doc CodeMirror.Doc 对象
 * @param {?} _erase
 * @param {?=} _multiple
 * @return {?}
 */
function image(doc, _erase, _multiple = false) {
    /** @type {?} */
    const start = doc.getCursor('start');
    /** @type {?} */
    const end = doc.getCursor('end');
    /** @type {?} */
    const selection = doc.getSelection();
    /** @type {?} */
    const from = { line: start.line, ch: start.ch + 2 };
    /** @type {?} */
    const to = { line: end.line, ch: end.ch + (start.line === end.line ? 2 : 0) };
    doc.replaceRange(`![${selection}](http://)`, start, end, selection);
    doc.setSelection(from, to, { scroll: false });
}
/**
 * 表格
 * @param {?} doc CodeMirror.Doc 对象
 * @param {?} _erase
 * @param {?=} _multiple
 * @return {?}
 */
function table(doc, _erase, _multiple = false) {
    /** @type {?} */
    let template = `
| Column 1 | Column 2 | Column 3 |
| -------- | -------- | -------- |
| Text     | Text     | Text     |
`;
    /** @type {?} */
    const end = doc.getCursor('end');
    doc.replaceRange(template, { line: end.line + 1, ch: 0 });
}
/**
 * 加粗
 * @param {?} doc CodeMirror.Doc 对象
 * @param {?} _erase
 * @param {?=} _multiple
 * @return {?}
 */
function bold(doc, _erase, _multiple = false) {
    /** @type {?} */
    const start = doc.getCursor('start');
    /** @type {?} */
    const end = doc.getCursor('end');
    /** @type {?} */
    const selection = doc.getSelection() || '';
    /** @type {?} */
    const from = { line: start.line, ch: start.ch + 2 };
    /** @type {?} */
    const to = { line: end.line, ch: end.ch + (start.line === end.line ? 2 : 0) };
    doc.replaceRange(`**${selection}**`, start, end, selection);
    doc.setSelection(from, to, { scroll: false });
}
/**
 * 斜体
 * @param {?} doc CodeMirror.Doc 对象
 * @param {?} _erase
 * @param {?=} _multiple
 * @return {?}
 */
function italic(doc, _erase, _multiple = false) {
    /** @type {?} */
    const start = doc.getCursor('start');
    /** @type {?} */
    const end = doc.getCursor('end');
    /** @type {?} */
    const selection = doc.getSelection() || '';
    /** @type {?} */
    const from = { line: start.line, ch: start.ch + 1 };
    /** @type {?} */
    const to = { line: end.line, ch: end.ch + (start.line === end.line ? 1 : 0) };
    doc.replaceRange(`*${selection}*`, start, end, selection);
    doc.setSelection(from, to, { scroll: false });
}
/**
 * 删除线
 * @param {?} doc CodeMirror.Doc 对象
 * @param {?} _erase
 * @param {?=} _multiple
 * @return {?}
 */
function strikethrough(doc, _erase, _multiple = false) {
    /** @type {?} */
    const start = doc.getCursor('start');
    /** @type {?} */
    const end = doc.getCursor('end');
    /** @type {?} */
    const selection = doc.getSelection() || '';
    /** @type {?} */
    const from = { line: start.line, ch: start.ch + 2 };
    /** @type {?} */
    const to = { line: end.line, ch: end.ch + (start.line === end.line ? 2 : 0) };
    doc.replaceRange(`~~${selection}~~`, start, end, selection);
    doc.setSelection(from, to, { scroll: false });
}
/** @type {?} */
const commands = {
    // 字体样式
    bold, italic, strikethrough,
    // header
    h1, h2, h3, h4, h5, h6,
    // list
    ul, ol,
    // midia
    link, image, table
};

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
class NtMarkdownEditorConfig {
    constructor() {
        this.actions = [
            ['bold', 'italic', 'strikethrough'],
            ['h1', 'h2', 'h3', 'h4', 'h5', 'h6'],
            // ['code', 'quote'],
            ['ul', 'ol'],
            ['link', 'image', 'table'],
            ['preview', 'help']
        ];
    }
}

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
/** @type {?} */
const DEFAULT_MIN_ROWS = 10;
/** @type {?} */
const DEFAULT_MAX_ROWS = 20;
/** @type {?} */
const MIN_ROWS_VALUE = 1;
/** @type {?} */
const MAX_ROWS_VALUE = 100;
class NtMarkdownEditorComponent extends NtFormFieldControl {
    /**
     * @param {?} _renderer
     * @param {?} _changeDetectorRef
     * @param {?} ngControl
     * @param {?} platformId
     * @param {?} icons
     */
    constructor(_renderer, _changeDetectorRef, ngControl, platformId, icons) {
        super();
        this._renderer = _renderer;
        this._changeDetectorRef = _changeDetectorRef;
        this.ngControl = ngControl;
        this.platformId = platformId;
        this.icons = icons;
        this._destroy = new Subject();
        this._syncScrollElements = [];
        this._syncScrollEvent = new Subject();
        this._value = '';
        this.previewMode = false;
        this.config = new NtMarkdownEditorConfig();
        this.activeActions = [];
        this.activeActionsSnapshot = [];
        this._minHeight = 100;
        this._maxHeight = 100;
        this._focused = false;
        this.placeholder = '';
        this._disabled = false;
        this._required = false;
        this._minRows = DEFAULT_MIN_ROWS;
        this._maxRows = DEFAULT_MAX_ROWS;
        this._theme = 'default';
        this._onChange = (/**
         * @return {?}
         */
        () => { });
        this._onTouched = (/**
         * @return {?}
         */
        () => { });
        if (this.ngControl) {
            this.ngControl.valueAccessor = this;
        }
        /** codemirror 在服务器环境下不支持以 es6 模块的方式导入，因此需要以动态加载的方式来处理。 */
        // if (isPlatformBrowser(this.platformId) && !codeMirrorLoaded) {
        //   fromTextArea = require('codemirror').fromTextArea;
        //   require('codemirror/mode/gfm/gfm');
        //   require('codemirror/addon/edit/continuelist');
        //   codeMirrorLoaded = true;
        // }
    }
    /**
     * @param {?} value
     * @return {?}
     */
    set value(value) {
        this._value = value;
        if (this.instance) {
            this.instance.setValue(value);
        }
    }
    /**
     * @return {?}
     */
    get value() { return this._value; }
    /**
     * @return {?}
     */
    get empty() { return !this.value; }
    /**
     * @return {?}
     */
    get focused() { return this._focused; }
    /**
     * @param {?} value
     * @return {?}
     */
    set disabled(value) { this._disabled = coerceBooleanProperty(value); }
    /**
     * @return {?}
     */
    get disabled() { return this._disabled; }
    /**
     * @return {?}
     */
    get required() { return this._required; }
    /**
     * @param {?} value
     * @return {?}
     */
    set required(value) { this._required = coerceBooleanProperty(value); }
    /**
     * @return {?}
     */
    get minRows() { return this._minRows; }
    /**
     * @param {?} value
     * @return {?}
     */
    set minRows(value) {
        /** @type {?} */
        const newValue = coerceNumberProperty(value) || 0;
        this._minRows = newValue < MIN_ROWS_VALUE ? MIN_ROWS_VALUE : newValue;
    }
    /**
     * @return {?}
     */
    get maxRows() { return this._maxRows; }
    /**
     * @param {?} value
     * @return {?}
     */
    set maxRows(value) {
        /** @type {?} */
        const newValue = coerceNumberProperty(value) || 0;
        this._maxRows = newValue > MAX_ROWS_VALUE ? MAX_ROWS_VALUE : newValue;
    }
    /**
     * @return {?}
     */
    get theme() { return this._theme; }
    /**
     * @param {?} value
     * @return {?}
     */
    set theme(value) { this._theme = value; }
    /**
     * @return {?}
     */
    get history() {
        if (this.instance) {
            /** @type {?} */
            const doc = this.instance.getDoc();
            return doc.historySize();
        }
        return { undo: 0, redo: 0 };
    }
    /**
     * @return {?}
     */
    ngOnInit() {
        if (isPlatformBrowser(this.platformId)) {
            this._codeMirrorInit();
        }
    }
    /**
     * @param {?} changes
     * @return {?}
     */
    ngOnChanges(changes) {
        /** @type {?} */
        const rowChanges = changes.minRows || changes.maxRows;
        if (rowChanges && !rowChanges.firstChange) {
            this._setEditorHeightRange();
        }
    }
    /**
     * @return {?}
     */
    ngOnDestroy() {
        this._destroy.next();
        this._destroy.complete();
        this._unsubscribeScrolls();
    }
    /**
     * @param {?} value
     * @return {?}
     */
    writeValue(value) {
        this.value = value;
    }
    /**
     * @param {?} fn
     * @return {?}
     */
    registerOnChange(fn) {
        this._onChange = fn;
    }
    /**
     * @param {?} fn
     * @return {?}
     */
    registerOnTouched(fn) {
        this._onTouched = fn;
    }
    /**
     * @return {?}
     */
    focus() {
        if (!this.disabled) {
            this.instance.focus();
        }
    }
    /**
     * @param {?} isDisabled
     * @return {?}
     */
    setDisabledState(isDisabled) {
        this._disabled = isDisabled;
    }
    /**
     * @param {?} type
     * @return {?}
     */
    execCommand(type) {
        if (type === 'preview') {
            this.previewMode = !this.previewMode;
            this._actionMatches();
            setTimeout((/**
             * @return {?}
             */
            () => {
                this.instance.refresh();
                this.previewMode ? this._subscribeScrolls() : this._unsubscribeScrolls();
            }));
        }
        else if (type === 'help') {
            window.open('https://guides.github.com/features/mastering-markdown/', '_blank');
        }
        else {
            /** @type {?} */
            const activeted = this.activeActions.indexOf(type) > -1;
            /** @type {?} */
            const doc = this.instance.getDoc();
            /** @type {?} */
            const start = doc.getCursor('start');
            /** @type {?} */
            const end = doc.getCursor('end');
            /** @type {?} */
            const multiple = end.line - start.line > 0;
            /** @type {?} */
            const command = commands[type];
            command && command(doc, activeted, multiple);
            this.instance.save();
            this.instance.focus();
        }
    }
    /**
     * @return {?}
     */
    undo() {
        this.instance.getDoc().undo();
    }
    /**
     * @return {?}
     */
    redo() {
        this.instance.getDoc().redo();
    }
    // 开始订阅滚动条同步事件
    /**
     * @private
     * @return {?}
     */
    _subscribeScrolls() {
        /** @type {?} */
        const callback = (/**
         * @param {?} event
         * @return {?}
         */
        (event) => {
            /** @type {?} */
            const element = (/** @type {?} */ (event.target));
            /** @type {?} */
            const scrollYUpdated = element.scrollTop !== element.previousScrollY;
            if (scrollYUpdated) {
                this._scrollTo(element);
            }
            element.previousScrollY = element.scrollTop;
        });
        /** @type {?} */
        const editorScrollElement = (/** @type {?} */ (this.instance.getScrollerElement()));
        this._syncScrollElements.push(editorScrollElement, this.markdown.nativeElement);
        this._syncScrollElements.forEach((/**
         * @param {?} element
         * @return {?}
         */
        element => element.addEventListener('scroll', element.callback = callback)));
        // 订阅同步滚动事件
        this._syncScrollSubscription = this._syncScrollEvent.subscribe((/**
         * @param {?} event
         * @return {?}
         */
        (event) => {
            this._syncScrollElements
                .filter((/**
             * @param {?} element
             * @return {?}
             */
            element => event.owner !== element))
                .forEach((/**
             * @param {?} element
             * @return {?}
             */
            element => {
                scrollY = element.previousScrollY = Math.round(event.rateY * (element.scrollHeight - element.clientHeight));
                if (Math.round(element.scrollTop - scrollY)) {
                    element.scrollTop = scrollY;
                }
            }));
        }));
        this._scrollTo(editorScrollElement);
    }
    // 取消订阅滚动条同步事件
    /**
     * @private
     * @return {?}
     */
    _unsubscribeScrolls() {
        if (this._syncScrollSubscription && !this._syncScrollSubscription.closed) {
            this._syncScrollSubscription.unsubscribe();
        }
        if (this._syncScrollElements) {
            this._syncScrollElements.forEach((/**
             * @param {?} element
             * @return {?}
             */
            element => element.removeEventListener('scroll', element.callback)));
            this._syncScrollElements = [];
        }
    }
    /**
     * @private
     * @param {?} element
     * @return {?}
     */
    _scrollTo(element) {
        this._syncScrollEvent.next({
            scrollY: element.scrollTop,
            rateY: element.scrollTop / (element.scrollHeight - element.clientHeight),
            owner: element
        });
    }
    /**
     * @private
     * @return {?}
     */
    _codeMirrorInit() {
        if (typeof fromTextArea === 'function') {
            this.editor.nativeElement.value = this.value;
            this.instance = fromTextArea(this.editor.nativeElement, {
                mode: 'gfm',
                lineWrapping: true,
                theme: this.theme || 'default',
                placeholder: this.placeholder,
                extraKeys: { Enter: 'newlineAndIndentContinueMarkdownList' }
            });
            this._setEditorHeightRange();
            this.instance.on('change', this._onCodeMirrorChange.bind(this));
            this.instance.on('cursorActivity', this._onCodeMirrorCursorActivity.bind(this));
            this.instance.on('focus', this._onCodeMirrorFocus.bind(this));
            this.instance.on('blur', this._onCodeMirrorBlur.bind(this));
            this._destroy.subscribe((/**
             * @return {?}
             */
            () => {
                this.instance.off('change', this._onCodeMirrorChange);
                this.instance.off('cursorActivity', this._onCodeMirrorCursorActivity);
                this.instance.off('focus', this._onCodeMirrorFocus);
                this.instance.off('blur', this._onCodeMirrorBlur);
                this.instance.toTextArea();
                this._changeDetectorRef.markForCheck();
            }));
            Promise.resolve().then((/**
             * @return {?}
             */
            () => this.instance.refresh()));
        }
    }
    /**
     * @private
     * @param {?} instance
     * @return {?}
     */
    _onCodeMirrorChange(instance) {
        this._actionMatches();
        this._value = instance.getValue();
        this._onChange(this._value);
        this._onTouched();
        this._changeDetectorRef.markForCheck();
    }
    /**
     * @private
     * @return {?}
     */
    _onCodeMirrorCursorActivity() {
        this._actionMatches();
        this._changeDetectorRef.markForCheck();
    }
    /**
     * @private
     * @return {?}
     */
    _onCodeMirrorFocus() {
        this._focused = true;
        this._changeDetectorRef.markForCheck();
    }
    /**
     * @private
     * @return {?}
     */
    _onCodeMirrorBlur() {
        this._focused = false;
        this._changeDetectorRef.markForCheck();
    }
    /**
     * @private
     * @return {?}
     */
    _actionMatches() {
        /** @type {?} */
        const doc = this.instance.getDoc();
        /** @type {?} */
        const start = doc.getCursor('start');
        /** @type {?} */
        const end = doc.getCursor('end');
        /** @type {?} */
        const range = doc.getRange({ line: start.line, ch: 0 }, end);
        this.activeActions = [];
        if (range.match(/^#\s/gm)) {
            this.activeActions.push('h1');
        }
        if (range.match(/^#{2}\s/gm)) {
            this.activeActions.push('h2');
        }
        if (range.match(/^#{3}\s/gm)) {
            this.activeActions.push('h3');
        }
        if (range.match(/^#{4}\s/gm)) {
            this.activeActions.push('h4');
        }
        if (range.match(/^#{5}\s/gm)) {
            this.activeActions.push('h5');
        }
        if (range.match(/^#{6}\s/gm)) {
            this.activeActions.push('h6');
        }
        if (range.match(/^[*\-+]\s/gm)) {
            this.activeActions.push('ul');
        }
        if (range.match(/^\d+\.\s+/gm)) {
            this.activeActions.push('ol');
        }
        if (this.previewMode) {
            this.activeActions.push('preview');
        }
    }
    /**
     * 以文本行数来计算输入窗口的高度。
     * @private
     * @return {?}
     */
    _setEditorHeightRange() {
        if (isPlatformBrowser(this.platformId) && !!this.instance) {
            /** @type {?} */
            const scrollElement = this.instance.getScrollerElement();
            /** @type {?} */
            const style = window.getComputedStyle(scrollElement);
            if (style && style.lineHeight) {
                /** @type {?} */
                const lineHeight = parseFloat(style.lineHeight.replace('px', ''));
                this._minHeight = this.minRows * lineHeight;
                this._maxHeight = this.maxRows * lineHeight;
                this._renderer.setStyle(scrollElement, 'min-height', `${this._minHeight}px`);
                this._renderer.setStyle(scrollElement, 'max-height', `${this._maxHeight}px`);
            }
        }
    }
}
NtMarkdownEditorComponent.decorators = [
    { type: Component, args: [{
                selector: 'nt-markdown-editor',
                template: "<div class=\"nt-markdown-editor-tools\" [class.tools-disable]=\"previewMode\"> <div class=\"tools-group\" *ngFor=\"let group of config.actions\"> <div class=\"tools-item\" *ngFor=\"let action of group\" [class.active]=\"activeActions.indexOf(action) > -1\" (click)=\"execCommand(action)\"> <span [class]=\"icons[action]\"></span> </div> </div> <div class=\"tools-group\"> <div class=\"tools-item\" [class.disabled]=\"history.undo === 0\" (click)=\"undo()\"> <span class=\"fa fa-reply\"></span> </div> <div class=\"tools-item\" [class.disabled]=\"history.redo === 0\" (click)=\"redo()\"> <span class=\"fa fa-share\"></span> </div> </div> </div> <div class=\"nt-markdown-editor-container\"> <div class=\"nt-markdown-editor-content\"> <textarea #editor></textarea> </div> <div #markdown class=\"nt-markdown-editor-preview\" *ngIf=\"previewMode\" [style.minHeight.px]=\"_minHeight\" [style.maxHeight.px]=\"_maxHeight\" [@fade]> <nt-markdown-block [data]=\"value\"></nt-markdown-block> </div> </div> ",
                encapsulation: ViewEncapsulation.None,
                changeDetection: ChangeDetectionStrategy.OnPush,
                providers: [
                    { provide: NtFormFieldControl, useExisting: NtMarkdownEditorComponent }
                ],
                animations: [
                    trigger('fade', [
                        transition('void => *', fadeIn(.15))
                    ])
                ],
                host: {
                    'class': 'nt-markdown-editor nt-form-control',
                    '[class.focus]': 'focused'
                }
            },] },
];
/** @nocollapse */
NtMarkdownEditorComponent.ctorParameters = () => [
    { type: Renderer2 },
    { type: ChangeDetectorRef },
    { type: NgControl, decorators: [{ type: Self }, { type: Optional }] },
    { type: Object, decorators: [{ type: Inject, args: [PLATFORM_ID,] }] },
    { type: undefined, decorators: [{ type: Inject, args: [NT_MARKDOWN_EDITOR_ICONS,] }] }
];
NtMarkdownEditorComponent.propDecorators = {
    editor: [{ type: ViewChild, args: ['editor', { static: true },] }],
    markdown: [{ type: ViewChild, args: ['markdown', { static: false, read: ElementRef },] }],
    value: [{ type: Input }],
    placeholder: [{ type: Input }],
    disabled: [{ type: Input }],
    required: [{ type: Input }],
    minRows: [{ type: Input }],
    maxRows: [{ type: Input }],
    theme: [{ type: Input }]
};

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
/** @type {?} */
const DEFAULT_MARKDOWN_EDITOR_ICONS = {
    bold: 'fa fa-bold',
    italic: 'fa fa-italic',
    strikethrough: 'fa fa-strikethrough',
    h1: 'fa fa-heading fa-heading-x fa-heading-1',
    h2: 'fa fa-heading fa-heading-x fa-heading-2',
    h3: 'fa fa-heading fa-heading-x fa-heading-3',
    h4: 'fa fa-heading fa-heading-x fa-heading-4',
    h5: 'fa fa-heading fa-heading-x fa-heading-5',
    h6: 'fa fa-heading fa-heading-x fa-heading-6',
    code: 'fa fa-code',
    quote: 'fa fa-quote-left',
    ul: 'fa fa-list-ul',
    ol: 'fa fa-list-ol',
    link: 'fa fa-link',
    image: 'fa fa-image',
    table: 'fa fa-table',
    line: 'fa fa-minus',
    preview: 'fas fa-columns',
    help: 'fab fa-markdown',
};
const ɵ0 = DEFAULT_MARKDOWN_EDITOR_ICONS;
class NtMarkdownEditorModule {
    /**
     * @param {?=} icons
     * @return {?}
     */
    static forRoot(icons = DEFAULT_MARKDOWN_EDITOR_ICONS) {
        return {
            ngModule: NtMarkdownEditorModule,
            providers: [
                { provide: NT_MARKDOWN_EDITOR_ICONS, useValue: icons }
            ]
        };
    }
}
NtMarkdownEditorModule.decorators = [
    { type: NgModule, args: [{
                imports: [CommonModule, NtMarkdownBlockModule],
                exports: [NtMarkdownEditorComponent],
                declarations: [NtMarkdownEditorComponent],
                providers: [
                    { provide: NT_MARKDOWN_EDITOR_ICONS, useValue: ɵ0 }
                ]
            },] },
];

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */

export { NtMarkdownEditorModule, NtMarkdownEditorComponent, NT_MARKDOWN_EDITOR_ICONS };
//# sourceMappingURL=markdown-editor.js.map
